#!/usr/bin/env python
# Author: (Tin Tran)
# Created On: 2024.06.21
# My own grunty bot/script that creates sheared patterns that look more organic
#
# License: Whatever GIMP's License is.
# Revisions:
# 0.1 Initial Version
from gimpfu import *
import random
import math
customset = [8,21,42] #reds
customset = [8,21,42,19,25,26,27] #reds & whites these are indexes of patterns starting from 0.

def shearedbrushes(image,layer, width, height, minshearx, maxshearx, minsheary, maxsheary, movexpercent, moveypercent, alternateweave):
    w = int(width) #previous we set these hardcode now being passed in for new image
    h = int(height)
    xtilesw = (layer.width*(movexpercent/100.0))
    ytilesh = (layer.height*(moveypercent/100.0))
    xtiles = int(math.ceil(w/xtilesw))
    ytiles = int(math.ceil(h/ytilesh))
    new_image = pdb.gimp_image_new(w,h,RGB)
    new_display = pdb.gimp_display_new(new_image)

    #numbrushes,brushlist = pdb.gimp_brushes_get_list("")

    numpat,patlist = pdb.gimp_patterns_get_list("")

    # override customset if we want all patterns below starting with 1 (2nd) since 0 (1st) is just current pattern on clipboard 
    # =================
    customset = []
    for i in range(1,len(patlist)):
        customset.append(i)
    # override customset ends    

    for y in range(-1,ytiles):
        for x in range(-2,xtiles):
            #create copy of layer
            #layer = image.layers[random.randrange(0,len(image.layers))] #grab a random layer
            layer_copy = pdb.gimp_layer_new_from_drawable(layer,new_image)
            pdb.gimp_image_insert_layer(new_image,layer_copy,None,0)

            pdb.gimp_context_set_pattern(patlist[customset[random.randrange(0,len(customset))]])
            pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,50,50,1024,1024)
            pdb.gimp_edit_fill(layer_copy,FILL_PATTERN)
            pdb.gimp_selection_none(new_image)


            #threads layers with transparency =========== drawn on top our layer after 
            layert = image.layers[random.randrange(0,len(image.layers)-1)] #grab a random layer
            layer_copy = pdb.gimp_layer_new_from_drawable(layert,new_image)
            pdb.gimp_image_insert_layer(new_image,layer_copy,None,0)
            layer_copy = pdb.gimp_image_merge_down(new_image,layer_copy,EXPAND_AS_NECESSARY)

            # ======= One off use case when I wanted to use gimp's brushes instead of patterns
            # ------- patterns are way more fun ---------------- or nice looking but who knows maybe we can use brushes if 
            # -------------------------------------------------- i was more into brushes ---------------------------------
            # pdb.gimp_context_set_brush(brushlist[random.randrange(0,len(brushlist))])
            # pdb.gimp_context_set_brush_size(1024)
            # pdb.gimp_paintbrush(layer_copy,0,2,[512,512],PAINT_CONSTANT,0)

            # this will offset the rows so that every alternating row will shift by 50% half so that it's even more organic
            if int(alternateweave)==1 and y%2==0:
                offsetx = xtilesw/2.0
            else:
                offsetx = 0

            #these below rshift values will shift randomly around so that it's more organic looking
            # =====================================================================================    
            rshiftx = random.randrange(0,int(xtilesw)) 
            rshifty = random.randrange(0,int(ytilesh))
            pdb.gimp_layer_set_offsets(layer_copy,rshiftx+x*xtilesw+offsetx,rshifty+h-layer.height-y*ytilesh)

            #SHEARING LOGICS HERE =======
            pdb.gimp_item_transform_shear(layer_copy,ORIENTATION_HORIZONTAL,random.randrange(int(minshearx),int(maxshearx)))
            pdb.gimp_item_transform_shear(layer_copy,ORIENTATION_VERTICAL,random.randrange(int(minsheary),int(maxsheary)))

register(
    "python_fu_shearedbrushes",
    "description",
    "longer description",
    "author name",
    "copyright name",
    "2024.06.21",
    "Sheared Brushes...",
    "RGB*",      # Alternately use RGB, RGB*, GRAY*, INDEXED etc.
    [
    #INPUT BEGINS
    (PF_IMAGE, "image", "Image", None),
    (PF_DRAWABLE,   "layer", "Drawable", None),
    (PF_SPINNER, "w", "New Image Width:", 1024*8, (0, 10000, 1)), #change this to be smaller in most cases
    (PF_SPINNER, "h", "New Image Height:", 1024*8, (0, 10000, 1)),
    (PF_SPINNER, "randshearx", "Min. Shear-X:", 0, (-1000, 1000, 1)),
    (PF_SPINNER, "randshearx", "Max. Shear-X:", 400, (-1000, 1000, 1)),
    (PF_SPINNER, "randsheary", "Min. Shear-Y:", -100, (-1000, 1000, 1)),
    (PF_SPINNER, "randsheary", "Max. Shear-Y:", 100, (-1000, 1000, 1)),
    (PF_SPINNER, "movexpercent", "Move X-percent:", 50, (0, 100, 1)),
    (PF_SPINNER, "moveypercent", "Move Y-percent:", 50, (0, 100, 1)),
    (PF_TOGGLE,  "alternateweave",   "Alternate Weave (Every Other Row):", 1),
    #INPUT ENDS
    ],
    [],
    shearedbrushes,
    menu="<Image>/Python-Fu")

main()

# Below is all the example input types for INPUTS for the plug-in which can be cut and pasted into #INPUT BEGINS section and edited to taste (found online years back that I didn't want to constantly look up)
# Since GIMP is free anyways, I thought it would be handy here for me to CUT and PASTE, CHANGE, USE.
#           (PF_INT, "p0", "_INT:", 0), # PF_INT8, PF_INT16, PF_INT32  similar but no difference in Python.
#           (PF_FLOAT, "p02", "_FLOAT:", 3.141),
#           (PF_STRING, "p03", "_STRING:", "foo"),  # alias PF_VALUE
#           (PF_TEXT, "p04", "TEXT:", "bar"),
#           # PF_VALUE
#           # Pick one from set of choices
#           (PF_OPTION,"p1",   "OPTION:", 0, ["0th","1st","2nd"]), # initially 0th is choice
#           (PF_RADIO, "p16", "RADIO:", 0, (("0th", 1),("1st",0))), # note bool indicates initial setting of buttons
#           # PF_RADIO is usually called a radio button group.
#           # SLIDER, ADJUSTMENT types require the extra parameter of the form (min, max, step).
#           (PF_TOGGLE, "p2",   "TOGGLE:", 1), # initially True, checked.  Alias PF_BOOL
#           # PF_TOGGLE is usually called a checkbox.
#           (PF_SLIDER, "p3", "SLIDER:", 0, (0, 100, 10)),
#           (PF_SPINNER, "p4", "SPINNER:", 21, (1, 1000, 50)),  # alias PF_ADJUSTMENT
#           # Pickers ie combo boxes ie choosers from lists of existing Gimp objects
#           (PF_COLOR, "p14", "_COLOR:", (100, 21, 40) ), # extra param is RGB triple
#           # PF_COLOUR is an alias by aussie PyGimp author lol
#           (PF_IMAGE, "p15", "IMAGE:", None), # should be type gimp.image, but None works
#           (PF_FONT, "p17", "FONT:", 0),
#           (PF_FILE, "p18", "FILE:", 0),
#           (PF_BRUSH, "p19", "BRUSH:", 0),
#           (PF_PATTERN, "p20", "PATTERN:", 0),
#           (PF_GRADIENT, "p21", "GRADIENT:", 0),
#           (PF_PALETTE, "p22", "PALETTE:", 0),
#           (PF_LAYER, "p23", "LAYER:", None),
#           (PF_CHANNEL, "p24", "CHANNEL:", None),  # ??? Usually empty, I don't know why.
#           (PF_DRAWABLE, "p25", "DRAWABLE:", None),
#           # Mostly undocumented, but work
#           (PF_VECTORS, "p26", "VECTORS:", None),
#           (PF_FILENAME, "p27", "FILENAME:", 0),
#           (PF_DIRNAME, "p28", "DIRNAME:", 0)
#           # PF_REGION might work but probably of little use.  See gimpfu.py.
